/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.commons;

import org.exoplatform.web.application.ApplicationMessage;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.exception.MessageException;
import org.exoplatform.webui.form.UIForm;
import org.exoplatform.webui.form.UIFormInput;
import org.exoplatform.webui.form.validator.Validator;

/**
 * Validates whether a field is empty This class acts like a flag "mandatory".
 * When you want to specify that a UIFormInput field is mandatory in your form,
 * add it this validator. A '*' character will be automatically added during the
 * rendering phase to specify the user
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 */
public class MandatoryValidator implements Validator {

  @SuppressWarnings("unchecked")
  public void validate(UIFormInput uiInput) throws Exception {
    if ((uiInput.getValue() != null) && ((String) uiInput.getValue()).trim().length() > 0) {
      return;
    }

    // modified by Pham Dinh Tan
    UIComponent uiComponent = (UIComponent) uiInput;
    UIForm uiForm = uiComponent.getAncestorOfType(UIForm.class);
    String label;
    try {
      label = uiForm.getLabel(uiInput.getName());
    } catch (Exception e) {
      label = uiInput.getName();
    }
    label = label.trim();
    if (label.charAt(label.length() - 1) == ':') {
      label = label.substring(0, label.length() - 1);
    }
    Object[] args = {label, uiInput.getBindingField()};
    throw new MessageException(new ApplicationMessage("EmptyFieldValidator.msg.empty-input", args, ApplicationMessage.WARNING));
  }
}
