/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.commons;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import org.exoplatform.portal.application.PortalRequestContext;
import org.exoplatform.portal.webui.util.Util;

/**
 * NovaData encapsulate data in order to be accessed by UINovaGrid.
 * 
 * @author Le Jeune Goulven <goulven.le-jeune@bull.net> 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
public class NovaData {

  /**
   * possible actions on the nova data
   */
  private ArrayList<NovaAction> action;

  /**
   * values of the different fields of the nova data
   */
  private HashMap<String, Object> data;

  /**
   * business entity that the nova data represent
   */
  private Object o;

  /**
   * Separator of a collection items
   */
  public static final String ITEM_SEPARATOR = " , ";

  public NovaData(Object o) {
    this.action = new ArrayList<NovaAction>();
    this.data = new HashMap<String, Object>();
    this.o = o;
  }

  /**
   * This method get the value for the given key (field) of the nova data, if
   * this value has already been set it return the known value else it invoke
   * with the given key (field) a getter method of this field on the nova data.
   * 
   * @param the field of the wanted value of the nova data
   * @return the value of the wanted nova data field
   * 
   * @throws SecurityException
   * @throws NoSuchMethodException
   * @throws IllegalArgumentException
   * @throws IllegalAccessException
   * @throws InvocationTargetException
   * 
   */
  @SuppressWarnings("unchecked")
  public String get(String key) throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
    String out = null;
    Object object = null;
    if (this.data.containsKey(key)) {
      object = this.data.get(key);
    } else {
      StringBuilder sb = new StringBuilder(key);
      sb.replace(0, 1, sb.substring(0, 1).toUpperCase());
      Method m = o.getClass().getMethod("get" + sb.toString(), (Class[]) null);
      object = m.invoke(o, (Object[]) null);
    }
    if (object == null) {
      out = null;
    } else if (object instanceof Date) {
      // format
      PortalRequestContext prContext = Util.getPortalRequestContext();
      SimpleDateFormat formattedDate = new SimpleDateFormat("dd MMMM yyyy HH:mm:ss", prContext.getLocale());
      out = formattedDate.format((Date) object);
    } else if (object instanceof Collection) {
      Iterator iterator = Collection.class.cast(object).iterator();
      out = "";
      boolean separator = false;
      StringBuffer buf = new StringBuffer();
      while (iterator.hasNext()) {
        if (separator) {
          buf.append(NovaData.ITEM_SEPARATOR);
        }
        buf.append(iterator.next().toString());
        separator = true;
      }
      out = buf.toString();
    } else {
      out = object.toString();
    }
    return out == null ? "" : out;
  }

  /**
   * This method allow to set a value of a field for the nova data
   * 
   * @param the field of the nova data
   * @param the value of the nova data field
   */
  public void set(String key, Object value) {
    data.put(key, value);
  }

  /**
   * This method return the possible actions for the nova data. If no action it
   * return null.
   * 
   * @return array of nova actions
   */
  public NovaAction[] getActions() {
    if (this.action.size() == 0) {
      return null;
    }
    return this.action.toArray(new NovaAction[1]);
  }

  /**
   * This method allow to add an action to the nova data
   * 
   * @param the nova action to add
   */
  public void addAction(NovaAction action) {
    this.action.add(action);
  }

  /*
   * =================== check if it's useful ======================== public
   * void addAction(String event, String header) { this.action.add(new
   * NovaAction(event,header)); }
   * 
   * public void addAction(String header,String event,String[] params) {
   * this.action.add(new NovaAction(header,event,params)); }
   * ==================================================================
   */
}
