/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA. 
 **/

package org.ow2.orchestra.console.webui.commons;

import org.exoplatform.web.application.ApplicationMessage;
import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.application.portlet.PortletRequestContext;
import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.core.UIApplication;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.core.lifecycle.UIFormLifecycle;
import org.exoplatform.webui.form.UIForm;

/**
 * This component upload a BAR file then deploy the process
 * 
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 */

@ComponentConfig(lifecycle = UIFormLifecycle.class, template = "system:/groovy/novabpm/commons/webui/UploadForm.gtmpl")
public abstract class UINovaUpload extends UIForm {

  public static final String FIELD_NAME = "name";
  public static final String FIELD_UPLOAD = "upload";

  /** Constructor **/
  public UINovaUpload() throws Exception {
    setMultiPart(true);
    UINovaUploadInput uiInput = new UINovaUploadInput(FIELD_UPLOAD, FIELD_UPLOAD);
    addUIFormInput(uiInput);
  }

  /**
   * This method update the ui upload component. It allow to give a new upload
   * identificator when an upload is done.
   */
  public void updateComponent() {
    UIApplication uiApp = getAncestorOfType(UIApplication.class);
    try {

      UIComponent input = getChild(UINovaUploadInput.class);
      removeChildById(input.getId());

      UINovaUploadInput uiInput = new UINovaUploadInput(FIELD_UPLOAD, FIELD_UPLOAD);
      addUIFormInput(uiInput);

    } catch (Exception e) {
      String[] error = {
        e.getMessage()
      };
      uiApp.addMessage(new ApplicationMessage("skip", error, ApplicationMessage.ERROR));
      ((WebuiRequestContext) WebuiRequestContext.getCurrentInstance()).addUIComponentToUpdateByAjax(uiApp.getUIPopupMessages());
    }
  }

  /**
   * Get the form identificator
   * 
   * @return form id
   */
  public String getFormId() {
    WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
    if (context instanceof PortletRequestContext) {
      return ((PortletRequestContext) context).getWindowId() + "#" + getId();
    }
    return getId();
  }
}
