/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.novaview;

import org.exoplatform.webui.Util;
import org.exoplatform.webui.application.ConfigurationManager;
import org.exoplatform.webui.application.WebuiApplication;
import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.config.Component;
import org.exoplatform.webui.core.UIComponent;

/**
 * UINovaViewFactory
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
public class UINovaViewFactory extends UIComponent {

  /**
   * This method allow to create a nova view
   * 
   * @param type of the view to create
   * @param config id
   * @param component id
   * @return the created nova view
   * @throws Exception
   */
  public UINovaView createView(Class< ? extends UINovaView> type, String configId, String componentId) throws Exception {
    UINovaView view = null;
    try {
      view = createUIComponent(type, configId, componentId);
    } catch (Exception e) {
      // the component can't be created because the configId is not on the
      // component.
      // We will search the config in the ancestor of the ViewManagement and
      // create the view with it.

      // Get the Configuration Manager
      WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
      WebuiApplication app = (WebuiApplication) context.getApplication();
      ConfigurationManager configManager = app.getConfigurationManager();

      // The ancestor of ViewManager is allways the parent of the its parent
      UIComponent parent = ((UIComponent) getParent()).getParent();
      Component config = configManager.getComponentConfig(parent.getClass(), configId);

      if (config == null) {
        // Ok the configuration is not defined, propagate the exception
        throw e;
      }
      view = Util.createObject(type, config.getInitParams());
      view.setComponentConfig(componentId, config);
      config.getUIComponentLifecycle().init(view, context);
    }
    return view;
  }

  /**
   * This method allow to create a nova view container
   * 
   * @param type of the view container to create
   * @param config id
   * @param component id
   * @return the created nova view container
   * @throws Exception
   */
  public UINovaViewContainer createViewContainer(Class< ? extends UINovaViewContainer> type, String configId, String componentId) 
    throws Exception {
    UINovaViewContainer component = null;
    try {
      component = createUIComponent(type, configId, componentId);
    } catch (Exception e) {
      // the component can't be created because the configId is not on the
      // component.
      // We will search the config in the ancestor of the ViewManagement and
      // create the view with it.

      // Get the Configuration Manager
      WebuiRequestContext context = WebuiRequestContext.getCurrentInstance();
      WebuiApplication app = (WebuiApplication) context.getApplication();
      ConfigurationManager configManager = app.getConfigurationManager();

      // The ancestor of ViewManager is allways the parent of the its parent
      UIComponent parent = ((UIComponent) getParent()).getParent();
      Component config = configManager.getComponentConfig(parent.getClass(), configId);

      if (config == null) {
        // Ok the configuration is not defined, propagate the exception
        throw e;
      }
      component = Util.createObject(type, config.getInitParams());
      component.setComponentConfig(componentId, config);
      config.getUIComponentLifecycle().init(component, context);
    }
    component.setFactory(this);
    return component;
  }
}
